PROGRAM Example1;
{
        This is an example of the SuperFX unit
        REALTIME version

        Copyright 1996,1997 by Just4Fun Productions

        A simple platform walking happening...
        No checking on "falling" of the screen, so once you fall, you have
        to restart by exiting to DOS and reloading.

        Features shown: - all directions scrolling
                        - 4 multiple layers of parallax
                        - infront "background" map tiles
                        - how to check on background collision
                          (just look at player.detection)
                        - screen zoom/sprite zoom
                        - fadein, fadeout
                        - the displacement procedure (see intro part)
                        - grey mode, or color mode (change status to SAD or
                          HAPPY)
                        - joystick/keyboard control
                        - joystick calibrating
                        - how to load the config file created with SETUP
                        - showing a HIGH screen image (360x480x256)

        Controls        : Use setup to select the movement controls,
                          CTRL to do a screen-wave
                          ALT  to flip the screen
                          M    to switch between SAD and HAPPY mode

}
{$IFDEF MSDOS}
   {$M 16384,0,5000}     { Minimum of HEAP memory needed }
{$ELSE}
   {$M 65520,0,655360}   { Maximum of HEAP memory needed }
{$ENDIF}

USES CRT,DOS,SuperFX,SFX_VARS;

CONST GameVersion = $003;               { game version                }
      CFGfile     = 'default.cfg';      { use the default config file }
{}
{

        Misc variables

}
TYPE StatusType = (Sad,Happy);
VAR  Status              : StatusType;
     StatusDelayPressing : byte;
{}
{

    Message Zooming variables

}
VAR   MessageZoomSize : byte;           { counter for size of zooming        }
      MessageNumber   : byte;           { number of the message sprite       }
      ZoomIT          : boolean;        { zooming into a sprite?             }
      MessageImages   : array[0..5] of SpriteTyp; { all the zoom messages    }
      flipY           : byte;
      flipA           : byte;
{}
{

    Player Ojbect - for PlatForm Games

}
TYPE TPlayerPlatForm   =  object
      DownPressed        : boolean;     { TRUE if pressed down to jump below }
      Xposition          : word;        { X position on the map in pixels }
      Yposition          : word;        { Y position on the map in pixels }
      Flipped            : boolean;     { Using mirrored images? }
      ground             : boolean;     { player walking on ground? }
      GroundFall         : boolean;     { if hit ground }
      Yspeed,Xspeed      : real;        { X and Y speed }

      score              : longint;     { take a guess?! }
      ScoreTXT           : string[9];   { the score is 9 digits }

      CurrentIMG         : byte; { Current animation frame }
        PROCEDURE Init;          { initialise the position, and variables }
        PROCEDURE Detection;     { detection of map-blocks (ground,wall,etc) }
        PROCEDURE Movement; {(tleft,tright,tup,tdown,tfire:boolean);}
        PROCEDURE AddScore(points:word); { add points to the score }
        PROCEDURE ShowScore(x,y:word);   { show the score at X,Y position }
     end;

VAR  PlayerImages : array[0..10] of SpriteTyp;{ the sprite-data of the player}
     Player       : TplayerPlatform;          { a complete PLAYER! }
{}
{
        Initialise the Player variables, and the map-position

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE TPlayerPlatform.Init;
BEGIN
    Xposition:=PlayerStartX shl TileShiftFactor;
    Yposition:=PlayerStartY shl TileShiftFactor;
    Flipped:=false;
    Ground:=false;
    GroundFall:=false;
    score:=000000000;
    ScoreTXT:='000000000';
    Xspeed:=0;
    Yspeed:=0;
    CurrentImg:=1;
    ASM
       mov ax,player.Xposition
       sub ax,160
       mov mapx,ax
       mov bx,player.Yposition
       sub bx,100
       mov mapY,bx
       Call Map_Calculate
    end;
    {

        See for ground below

    }
    with PLAYER do begin
         CASE mapPos((xposition shr TileShiftFactor)+1,yposition shr TileShiftFactor,mapBitFlag) of
         1..2 : BEGIN
                 Yposition:=Yposition shr TileShiftFactor;
                 inc(Yposition);
                 Yposition:=Yposition shl TileShiftFactor;
                 dec(Yposition,mem[seg(PlayerImages[0].data^):ofs(PlayerImages[0].data^)+1]);
                END;
         end;
    end;
END;
{}
{
        Add points to the score (score value, and scoreTEXT will be updated)

        Expects: Amount of points to increase the score with
        Returns: Nothing
}
PROCEDURE TPlayerPlatform.AddScore(points:word);
BEGIN
 inc(score,points);                     { increase score counter             }
 str(score,scoreTXT);                   { set it into a string only once     }
 fillchar(scoreTXT[1],9-length(scoreTXT),'0');   { add some zeros infront    }
 scoreTXT[0]:=char(9);                  { make sure it's only 9 characters   }
END;
{}
{
        Show the score of the player

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE TPlayerPlatform.ShowScore(x,y:word);
BEGIN
 WTEXT(x,y,scoreTXT,1);    { simply put the score on screen }
END;
{}
{
        Do detection on the player hitting background objects

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE TplayerPlatform.Detection;
VAR ty2,ty3,tx2,tx3   : word;
    x,y               : word;
    ctop              : boolean;
    cbot              : boolean;
    cright            : boolean;
    cleft             : boolean;
    cslope            : boolean;
    wi,hi             : byte;
    xt,yt,
    mapt              : integer;
    dataptr           : pointer;
    bitflag,f_Frame   : byte;
BEGIN
 ASM
  mov ax,player.yposition
  shr ax,TileshiftFactor
  cmp ax,0
  jng @nodecreaseY
  dec ax
 @nodecreaseY:
  mov ty2,ax
  add ax,2
  mov ty3,ax
  mov ax,player.xposition
  shr ax,TileShiftFactor
  cmp ax,0
  jng @nodecreaseX
  dec ax
 @nodecreaseX:
  mov tx2,ax
  add ax,2
  mov tx3,ax

  mov cslope,false
  mov ctop,false
  mov cbot,false
  mov cleft,false
  mov cright,false
  mov cleft,false
 END;

 { Get width and height of player image }
 wi:=mem[seg(PlayerImages[currentIMG].data^):ofs(PlayerImages[currentIMG].data^)];
 hi:=mem[seg(PlayerImages[currentIMG].data^):ofs(PlayerImages[currentIMG].data^)+1];

 FOR x:=tx2 to tx3 do begin
   xt:=x shl TileShiftFactor; { calculate X shl 5 only onces per vertical Row! }
   FOR y:=ty2 to ty3 do begin
   {==========================================================================
        Now get the blocks above,below,left and on the right side of the
        player
   ==========================================================================}
       BitFlag:=mapPos(x,y,mapBitflag);
       f_frame:=mapPos(x,y,mapF_frame);
        yt:=y shl TileShiftFactor;
        {=====================================================================
            See if this block is not just empty-air
        =====================================================================}
        { Some testing now... }
        IF BITFLAG<>0 THEN BEGIN
           {==================================================================
              Slope testing
           ==================================================================}
           if (yspeed>=0.0) and (collision( xt,yt,
                          mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)],
                          mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)+1],
                          xposition+15,yposition,1,hi))
                          then begin
                             CASE BITFLAG of
                              4 : begin   { slope like  \  }
                                    Yposition:=1+yt-hi+( (( (Xposition+15)-xt) shl 1)+1) shr 1;
                                    Yspeed:=0;
                                    CBot:=true;
                                    ground:=true;
                                  end;
                              5 : begin   { slope like  / }
                                    Yposition:=1+yt-hi+32-( (( (Xposition+15)-xt) shl 1)+1) shr 1;
                                    CBot:=true;
                                    Yspeed:=0;
                                    ground:=true;
                                  end;
                             END; { end case }
           end;

           {==================================================================
              See player position (block behind the player)
           ==================================================================}
           if (collision( xt,yt,
                          mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)],
                          mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)+1],
                          xposition,yposition+1,wi,hi))
                          then begin
                             CASE BITFLAG of
                             { ********************
                                 Secret IDentifier
                             **********************}
                                  9 : begin
                                         if not zoomIT then begin
                                            ZoomIT:=true;
                                            MessageZoomSize:=2;
                                            MessageNumber:=1;
                                         end;
                                      end;

                             end; { case blocktype }
           end;

           {==================================================================
              See Top of player
           ==================================================================}
           IF NOT CTOP then
           if (collision( xt+5,
                          yt+mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)+1]-5,
                          mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)]-10,5,
                          xposition,yposition,wi,10))
                          then begin
                             CASE BITFLAG of
                             { *****************
                                 Massive Block
                             ******************}
                                  1 : begin
                                         yposition:=1+yt+mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)+1];
                                         yspeed:=0.2;
                                         ground:=false;
                                         ctop:=true;
                                      end;
                             end; { case blocktype }
           end;
           {==================================================================
              See Bottom of player
           ==================================================================}
           IF NOT cbot then
           if (yspeed>0.0) and  (collision( xt+5,yt-1,mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)]-10,7,
                           xposition,yposition+hi-5,wi,5))
                           then begin
                             CASE BITFLAG of
                             { **************
                                 Walk able
                             ****************}
                                  1,6: begin
                                          yposition:=yt-hi;
                                          Yspeed:=0;
                                          ground:=true;
                                          cbot:=true;
                                         end;
                                  2  : begin
                                         if not DownPressed then begin
                                            yposition:=yt-hi;
                                            Yspeed:=0;
                                            ground:=true;
                                            cbot:=true;
                                         end;
                                       end;
                             end; { case blocktype }
           end;
           {==================================================================
              See Right Side of the player
           ==================================================================}
           if not cright then
           if (xspeed>0) and (collision(xt,yt+5,5,mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)+1]-10,
                         xposition+wi,yposition,1,hi)) then begin
              CASE BITFLAG of
              {*******************
                   Massive Block
              ********************}
                1 : begin
                     xposition:=xt-wi;
                     xspeed:=0;
                     cright:=true;
                    end;
              end; { case blocktype }
           end;
           {==================================================================
              See Left Side of the player
           ==================================================================}
           if not cleft then
           if (xspeed<0) and
              (collision(xt,yt+5,mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)],
                         mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)+1]-10,
                         xposition,yposition,1,hi)) then begin
              CASE BITFLAG of
              {*******************
                  Massive Block
              ********************}
                1 : begin
                     xposition:=xt+mem[seg(tiles[f_Frame].data^):ofs(tiles[f_Frame].data^)];
                     xspeed:=0;
                     cleft:=true;
                    end;
              end; { case blocktype }
           end; { end left side }
    { --------------------------------------------------------------------- }
    END; { end collision_detection }
  END; { Y counter }
 END; { X counter }
END;
{}
{
        Do the player movements

        Expects: Nothing
        Returns: Nothing
}

PROCEDURE TPlayerPlatform.Movement;
VAR Tleft,TRight,TDown,Tup,TFire : boolean;
BEGIN
   Tleft:=JoystickLeft;
   Tright:=JoystickRight;
   TUp:=JoystickUp;
   TDown:=JoystickDown;
   TFire:=JoystickA;

   Ground:=false;
   Detection;
   if not GROUND then begin
      if yspeed<6 then yspeed:=yspeed+0.3;
   end else DownPressed:=false;
   WITH PLAYER DO BEGIN
        IF Tleft then begin
                 if not Flipped then Flipped:=true;
                 if xspeed>-4 then xspeed:=xspeed-0.2;
                 if ground then begin
                    if currentimg>3 then currentimg:=0;
                    currentIMG:=playerimages[currentimg].a_Frame;
                 end;
        end else IF Tright then begin
                 if FLIPPED then Flipped:=false;
                 if xspeed<4 then xspeed:=xspeed+0.2;
                 if ground then begin
                    if currentimg>3 then currentimg:=0;
                    currentIMG:=playerimages[currentimg].a_Frame;
                 end;
        end else begin
                 if xspeed>0.0 then xspeed:=xspeed-0.2
                 else
                   if xspeed<0.0 then xspeed:=xspeed+0.2
                 else xspeed:=0;
               CurrentIMG:=5;
        end;

        IF (round(xspeed)=0) and (round(Yspeed)=0) then begin
           CurrentIMG:=5;
           if Flipped then begin
              if mapPos((xposition shr TileShiftFactor)+1,(yposition shr TileShiftFactor)+1,mapBitFlag)=0 then inc(CurrentImg);
           end else begin
              if (mapPos((xposition shr TileShiftFactor)+1,(yposition shr TileShiftFactor)+1,mapBitFlag)=0) and
                 (xposition and 31>15) then inc(CurrentImg);
           end;
        end else
           if round(yspeed)<>0 then currentImg:=3;

        IF (TUp) then begin
           if (GROUND) then begin
              yspeed:=-7.6;
              GroundFall:=true;
           end;
        END;

        IF TDown then DownPressed:=true else
           if yspeed>4 then DownPressed:=false;

        Xposition:=Xposition+round(Xspeed);
        if (xposition-mapx)>294 then begin
           if xspeed<0 then xposition:=0 else xposition:=294+mapx;
           xspeed:=0;
        end;
        Yposition:=Yposition+round(Yspeed);
        asm
         mov ax,player.Xposition
         sub ax,155
         mov mapx,ax
         mov bx,player.Yposition
         sub bx,95
         mov mapY,bx
         Call Map_Calculate
        end;
        AddIMAGE( Player.Xposition-MAPX,Player.Yposition-MAPY,
                  PlayerImages[CurrentImg].data,Flipped,false);
   END;
END;
{}
{
        Show the level start, in a very flashy way

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE PopLevel;
VAR Ypos,Yspeed  : integer;
    YInitSpd     : integer;
    dataptr      : pointer;
    temp_seg     : word;
    donefirstbounce : Boolean;
    xtarget         : integer;
BEGIN
 temp_seg:=vpage_Seg;   { remember current virtual page segment }
 {

   Initialise the screen

 }
 ClearScreen;
 ClearVisible;
 {

    Calculate the map

 }
 ASM
    mov ax,player.Xposition
    sub ax,155
    mov mapx,ax
    mov bx,player.Yposition
    sub bx,95
    mov mapY,bx
    Call Map_Calculate
 END;

 {

   Let the player run into the screen

 }
 xtarget:=mapx-player.xposition+90;
 player.xspeed:=4;
 REPEAT
   if xtarget<player.xposition-20 then begin
      player.currentimg:=playerimages[player.currentimg].a_Frame;
   end else player.currentIMG:=5;

   xtarget:=xtarget+round(player.xspeed);
   if xtarget>player.xposition-20 then
      if player.xspeed>0.4 then player.xspeed:=player.xspeed-0.4 else player.xspeed:=0;

   DataPtr:=PlayerImages[player.CurrentIMG].data;
   {

     Draw player image

   }
   ASM
    mov ax,xtarget
    mov bx,player.Yposition
    sub bx,MapY
    sub ax,MapX
    les DI,dataptr
    Call Sprite
   END;

   ShowBitmap;
   ClearScreen;
 UNTIL player.xspeed=0.0;
 {

     Draw the map

 }
 vpage_seg:=temp_seg;
 UpdateMap;
 DataPtr:=PlayerImages[PlayerImages[0].a_Frame+2].data;
 {

   Now bounce the screen in

 }
 Ypos:=199;
 Yspeed:=-25;
 YInitSpd:=-17;
 DoneFirstBounce:=false;

 REPEAT
    ASM
      xor ax,ax
      mov bx,Ypos
     call Displace
    END;
    {

       Now to show the player in the center (screen bouncing behind him)

    }
   vpage_seg:=sega000;    { set it to visible screen }
   IF Player.FLipped then { flipped image? }
      ASM
        mov ax,player.Xposition
        mov bx,player.Yposition
        sub bx,MapY
        sub ax,MapX
        les DI,dataptr
        Call XFlipSprite;
      END
   else
      ASM               { non flipped image }
        mov ax,player.Xposition
        mov bx,player.Yposition
        sub bx,MapY
        sub ax,MapX
        les DI,dataptr
        Call Sprite;
      END;
   vpage_seg:=temp_seg;  { get segment of virtual page back }

   inc(Ypos,Yspeed);
   inc(Yspeed);
   if ypos<0 then DoneFirstBounce:=true;
   if (ypos>=0) and (DoneFirstBounce) then begin
      ypos:=0;
      Yspeed:=YInitSpd;
      inc(YInitSpd,4);
   end;
 UNTIL (YInitSpd>=0) or (key[esckey]);
END;
{}
{
        Zoom a message-sprite

        Expects: Nothing
        Returns: Nothing
}
PROCEDURE ZoomMessage;
BEGIN
  ScaleShape( MessageImages[MessageNumber].data^,
              160-(MessageZoomsize shr 1),100-(MessageZoomsize shr 1),
              160+(MessageZoomsize shr 1),100+(MessageZoomsize shr 1));
  inc(MessageZoomsize,4);
  if MessageZoomsize>150 then begin
     MessageZoomsize:=2;
     ZoomIT:=false;
  end;
END;

{}
VAR i,j  : word;      { some variables to use with the zoom, and in-bounce }
{}
{       LET'S START THIS EXAMPLE!!!                                           }
BEGIN
 { read the loading-icon }
 UseFile('superfx.dfp');
 LoadIcon(true,'loadicon.xas');

 { parse the GCODE file for init }
 if not _ParseFile('example1.ini') then begin
    halt_Error:='File not found: example1.ini';
    halt;
 end;

 LoadCFG('default.cfg');

 GetXas(true,'message.xas',messageImages);{ read zoom messages               }
 GetXas(true,'mayhem.xas',PlayerImages);  { read main player images          }

 Player.Init;          { initialise the player                          }
 Status:=HAPPY;        { change this to SAD for a black/white version   }

 {=-=-=-= just show the startup screen =-=-=-=-=}
 writeln('Init done, press a key');
 repeat until anypressed;

 FadeOut(0,255);                          { fade out the screen              }
 InitVGAmode;                             { Initialise VGA mode              }
 LoadPal(true,'palette.col');             { load the palette                 }


 SETPAL;               { set the correct pal                            }
 if status=SAD then GreyPal(0,247); { if SAD mode, make it GREY         }

 InitJoy;      { this will initialise the joystick but only if needed   }

 {=-=-=-=-   Now, we start the level   =-=-=-=-=-=}
{ PopLevel;}
 {=-=-=-=-  Show the GO! message  =-=-=-=-=-=}
 MessageZoomsize:=2;   { start with size 2                              }
 ZoomIT:=true;         { zoomIT = true                                  }
 MessageNumber:=0;     { show message GO                                }

 AddLight(0,170,320,200,10,false,0);
 Init_Snow;


 REPEAT

  CASE ControlTYPE of
   1 : Player.Movement; {(key[left],key[right],key[up],key[down],key[ctrl]);}
   2 : BEGIN
          GetJoy;
          Player.Movement;{(Joystickleft,joystickright,joystickUp,joystickdown,JoystickB);}
   end;
  END;

  UpdateMap;
   Snow(1,1);
   {=-=-=-=-=-=  Now show the status bar stuff =-=-=-=-=-=-}
   Player.ShowScore(4,185);
   WTEXT (4,175,'SCORE',1);
   WTEXT(220,175,'TIME',1);
   WTEXT(280,175,'1UP',1);
   CASE Status of
    SAD   : WTEXT(140,175,'MAGIC',1);
    HAPPY : WTEXT(140,175,'STARS',1);
   END;

  if ZoomIT then ZoomMessage;

   if key[alt] then FlipSCreenY else
      if FXWave then WaveFX else ShowBitmap;

   IF StatusDelayPressing>0 then dec(StatusDelayPressing) else
      IF key[ 50{M} ] then begin
         case STATUS of
             HAPPY : begin
                       STATUS:=SAD;
                       GreyPal(0,247);
                       StatusDelayPressing:=30;
             end;
             SAD   : begin
                       STATUS:=HAPPY;
                       SetPal;
                       StatusDelayPressing:=30;
             end;
         end;
   END;

   IF key[CTRL] then InitWave(5);
     if key[41] then _Console_Pop;

 UNTIL (key[esckey]);

 {

   Zoom it out

 }
 i:=1;
 j:=2;
 REPEAT
    zoom(160,100,i);
    if j>0 then dec(j) else begin
       j:=2;
       inc(i);
    end;
 UNTIL i=15;
 FadeOut(0,255);

END.

